#!/usr/bin/env bash

shopt -s dotglob
[ "${DEBUG:-}" == 'true' ] && set -x

SERVER_CFG='server.cfg'

set_default ()
{
    server_ip='0.0.0.0'
    server_port=''
    server_hostname=''
    server_maxplayers=''
    server_license_key=''
}

parse_options ()
{
    for i in "$@"
    do
        case $i in
            -h|--help)
                show_logo
                show_help
                exit 0
            ;;
            --ip=*|--host=*)
                server_ip="${i#*=}"
                shift
            ;;
            --port=*)
                server_port="${i#*=}"
                shift
            ;;
            --rcon-password=*)
                server_rcon_password="${i#*=}"
                shift
            ;;
            --hostname=*)
                server_hostname="${i#*=}"
                shift
            ;;
            --maxplayers=*)
                server_maxplayers="${i#*=}"
                shift
            ;;
            --license-key=*)
                server_license_key="${i#*=}"
                shift
            ;;
        esac
    done
}

show_help ()
{
    echo
    echo 'Usage:	./fivem_run.sh COMMAND [OPTIONS]'
    echo
    echo 'Options:'
    echo '      --ip=                   Server ip'
    echo '      --port=                 Server port'
    echo '      --rcon-password=        Server rcon password'
    echo '      --hostname=             Server hostname'
    echo '      --maxplayers=           Server maxplayers'
    echo '      --license-key=          License key from https://keymaster.fivem.net'
    echo
    echo 'Examples:'
    echo '      ./fivem_run.sh --hostname="My Super Server" --ip=0.0.0.0 --port=25565 --license-key=myKeyFr0mkeymasterFiveMnEt'
}

show_logo ()
{
    echo
    echo '
                  ###           ####
               #######################
               #######################
              #######````````````#######
            ######   ############   ######
          #####  ####             ##  ########
      ########  ###               ####  ########
     ######## ###                 ###### ########
      ###### ####     ################### ######
       ####  ####     ################### #####
       #### #####     ###           #####  ####
       #### #####     ###           #####  ####
       ####  ####     #########     ##### #####
      ###### ####     #########     ##### ######
     ######## ###       ######      #### ########
      ########  ##                 ###  ########
       ########   ###            ###   #####
            #####    ############   ######
              #######````````````#######
                #######################
                #######################
                 #####          ####'
    echo
    echo '             GameAP FiveM Server Runner'
    echo
    echo '----------------------------------------------------'
    echo
}

set_properties ()
{
    if [ ! -f "${SERVER_CFG}" ]; then
        return
    fi

    if [ -n "${server_license_key:-}" ]; then
        sed -i "s/sv_licenseKey.*$/sv_licenseKey ${server_license_key}/" "${SERVER_CFG}"
    fi

    if [ -n "${server_maxplayers:-}" ]; then
        sed -i "s/sv_maxclients.*$/sv_maxclients ${server_maxplayers}/" "${SERVER_CFG}"
    fi

    if [ -n "${server_hostname:-}" ]; then
        sed -i "s/sv_hostname.*$/sv_hostname \"${server_hostname}\"/" "${SERVER_CFG}"
    fi

    if [ -n "${server_port:-}" ]; then
        sed -i "s/endpoint_add_tcp.*$/endpoint_add_tcp \"${server_ip}:${server_port}\"/" "${SERVER_CFG}"
        sed -i "s/endpoint_add_udp.*$/endpoint_add_udp \"${server_ip}:${server_port}\"/" "${SERVER_CFG}"
    fi

    if [ -n "${server_rcon_password:-}" ]; then
        sed -i "s/\#\?rcon_password.*$/rcon_password \"${server_rcon_password}\"/" "${SERVER_CFG}"
    fi
}

download ()
{
    artifacts_link=$(curl http://files.gameap.ru/fivem/latest-linux-artifacts)

    if ! curl -O "${artifacts_link}"; then
        echo 'Unable to download artifacts' >> /dev/stderr
        exit 1
    fi

    if ! tar -xf fivem.tar.xz --exclude=alpine/dev/null; then
        echo 'Unable to unpack artifacts' >> /dev/stderr
        exit 1
    fi

    rm fivem.tar.xz
    if ! curl -OL 'https://github.com/citizenfx/cfx-server-data/archive/master.zip'; then
        echo 'Unable to download cfx-server-data' >> /dev/stderr
        exit 1
    fi

    if ! unzip master.zip -d ./ ; then
        echo 'Unable to unzip cfx-server-data' >> /dev/stderr
        exit 1
    fi

    rm master.zip

    rsync -av --remove-source-files cfx-server-data-master/* ./
    rm -r cfx-server-data-master
}

run ()
{
    if [ ! -f ./run.sh ]; then
        download
    fi

    set_properties

    ./run.sh +exec server.cfg
}

main ()
{
    show_logo
    run
}

set_default
parse_options "$@"
main